@@ -21,6 +21,7 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111 USA
  */
 
+#include "config.h"
 #include <stdio.h>
 #include <unistd.h>
 #include <fcntl.h>
@@ -36,8 +37,14 @@
 #include "logger.h"
 #include "gui-handlers.h"
 #include "gui-config.h"
+#include "gui-sched.h"
 #include "pixmaps/unknown.xpm"
 
+
+#include <gtk/gtk.h>
+#include <gdk/gdkx.h>
+
+
 //#define DEBUG_CHEOPS_GUI
 
 #ifdef DEBUG_CHEOPS_GUI
@@ -55,10 +62,6 @@ net_page *current_page;
 /* global preferences */
 gui_preferences preferences;
 
-/* agent to talk to the cheops-agent */
-agent *primary_agent;
-char *agent_ip = NULL;
-
 /* this is for modal gnome_dialogs, so we can kill them if the upper level application is killed */
 GtkWidget *kill_me_too = NULL;
 
@@ -71,7 +74,7 @@ event *ee = (event *)(ebuf + sizeof(event_hdr));
 
 static void do_save(GtkWidget *widget, int data);
 void open_file_dialog(GtkWidget *w, gpointer p);
-void make_save_dialog(char *title, char *filename, void *ok_clicked_function, void *cancel_clicked_function);
+void make_save_dialog(char *title, char *filename, void *ok_clicked_function, void *cancel_clicked_function, char *message);
 static void do_write_config_file_and_exit(GtkWidget *w, GtkWidget *fs);
 
 
@@ -104,9 +107,9 @@ void save_changes(int quit)
 						options_save_changes_on_exit = TRUE;
 					
 					if(quit)
-						make_save_dialog("Save As", NULL, do_write_config_file_and_exit, NULL);
+						make_save_dialog("Save As", NULL, do_write_config_file_and_exit, NULL, NULL);
 					else
-						make_save_dialog("Save As", NULL, open_file_dialog, NULL);
+						make_save_dialog("Save As", NULL, open_file_dialog, NULL, NULL);
 						
 					return; // dont quit yet, open up a file save dialog
 					
@@ -145,13 +148,24 @@ void save_changes(int quit)
 
 void do_quit(GtkWidget *w, gpointer data)
 {
-
+	int save = 0;
+	net_page *np;
+	
 	if(kill_me_too != NULL)
 		gnome_dialog_close(GNOME_DIALOG(kill_me_too));
 
 	we_are_still_running = FALSE;
 
-	if(main_window->connected)
+	for(np = main_window->net_pages; np; np = np->next)
+	{
+		if(np->connected)
+		{
+			save = 1;
+			break;
+		}
+	}
+
+	if(save)
 	{
 		save_changes(TRUE);
 	}
@@ -180,7 +194,8 @@ void do_page(GtkWidget *widget, gint data)
 	GtkWidget *hbox;
 	GtkWidget *label;
 	GtkWidget *entry1;
-
+	net_page *np;
+	
 	switch(data)
 	{
 		case 0:
@@ -226,6 +241,53 @@ void do_page(GtkWidget *widget, gint data)
 				remove_viewspace(NULL, dialog);
 			}
 			break;
+		case 2:
+			dialog = gnome_dialog_new("Rename Viewspace",
+							  GNOME_STOCK_BUTTON_OK,
+							  GNOME_STOCK_BUTTON_CANCEL,
+							  NULL);
+			gnome_dialog_close_hides(GNOME_DIALOG(dialog), FALSE);
+			
+			entry1 = gnome_entry_new("new_viewspace");
+			label = gtk_label_new("Viewspace Name:");
+			hbox = gtk_hbox_new(FALSE, 5);
+			gnome_dialog_set_parent(GNOME_DIALOG(dialog), GTK_WINDOW(main_window->window));
+			
+			gtk_box_pack_start( GTK_BOX(hbox), label, FALSE, FALSE, 5);
+			gtk_box_pack_start( GTK_BOX(hbox), entry1, FALSE, FALSE, 5);
+			gtk_box_pack_start( GTK_BOX( GNOME_DIALOG(dialog)->vbox ), hbox, TRUE, TRUE, 0);
+
+			np = get_current_net_page();
+			if(np && np->label)
+			{
+				char *text = NULL;
+				gtk_label_get(GTK_LABEL(np->label), &text);
+				if(text != NULL)
+				{
+					gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(entry1)->entry), text);
+				}
+			}
+			gtk_widget_grab_focus(GTK_COMBO(entry1)->entry);
+
+			gtk_signal_connect(GTK_OBJECT(GTK_COMBO(entry1)->entry),  "activate", GTK_SIGNAL_FUNC(click_ok_on_gnome_dialog), dialog);
+			
+			gtk_widget_show( hbox );
+			gtk_widget_show( label );
+			gtk_widget_show( entry1 );
+
+			switch( gnome_dialog_run(GNOME_DIALOG(dialog)) )
+			{
+				case 0:
+					net_page_rename(np, gtk_entry_get_text(GTK_ENTRY(gnome_entry_gtk_entry(GNOME_ENTRY(entry1)))) );
+					gnome_dialog_close(GNOME_DIALOG(dialog));
+					break;
+				case 1:	
+					gnome_dialog_close(GNOME_DIALOG(dialog));
+					break;
+				default:
+					break;
+			}
+			break;
 		default:
 			break;
 	}
@@ -314,14 +376,15 @@ static void do_network(GtkWidget *widget, gint data)
 
 			gtk_widget_grab_focus(GTK_COMBO(entry1)->entry);
 
-			gtk_signal_connect(GTK_OBJECT(GTK_COMBO(entry2)->entry),  "activate", GTK_SIGNAL_FUNC(click_ok_on_gnome_dialog), dialog);
-			
 			gtk_widget_show( hbox );
 			gtk_widget_show( label );
 			gtk_widget_show( entry1 );
 			gtk_widget_show( label2 );
 			gtk_widget_show( entry2 );
 
+			gtk_signal_connect(GTK_OBJECT(GTK_COMBO(entry1)->entry),  "activate", GTK_SIGNAL_FUNC(gtk_widget_focus_me), GTK_COMBO(entry2)->entry);
+			gtk_signal_connect(GTK_OBJECT(GTK_COMBO(entry2)->entry),  "activate", GTK_SIGNAL_FUNC(click_ok_on_gnome_dialog), dialog);
+
 			switch( gnome_dialog_run(GNOME_DIALOG(dialog)) )
 			{
 				case 0:
@@ -366,16 +429,17 @@ static void do_network(GtkWidget *widget, gint data)
 
 			gtk_box_pack_start( GTK_BOX( GNOME_DIALOG(dialog)->vbox ), hbox, TRUE, TRUE, 0);
 
-			gtk_widget_grab_focus(GTK_COMBO(entry1)->entry);
-
-			gtk_signal_connect(GTK_OBJECT(GTK_COMBO(entry2)->entry),  "activate", GTK_SIGNAL_FUNC(click_ok_on_gnome_dialog), dialog);
-			
 			gtk_widget_show( hbox );
 			gtk_widget_show( label );
 			gtk_widget_show( entry1 );
 			gtk_widget_show( label2 );
 			gtk_widget_show( entry2 );
 
+			gtk_widget_grab_focus(GTK_COMBO(entry1)->entry);
+
+			gtk_signal_connect(GTK_OBJECT(GTK_COMBO(entry1)->entry),  "activate", GTK_SIGNAL_FUNC(gtk_widget_focus_me), GTK_COMBO(entry2)->entry);
+			gtk_signal_connect(GTK_OBJECT(GTK_COMBO(entry2)->entry),  "activate", GTK_SIGNAL_FUNC(click_ok_on_gnome_dialog), dialog);
+			
 			switch( gnome_dialog_run(GNOME_DIALOG(dialog)) )
 			{
 				case 0:
@@ -465,9 +529,10 @@ static void do_write_config_file_and_exit(GtkWidget *w, GtkWidget *fs)
 }
 
 
-void make_save_dialog(char *title, char *filename, void *ok_clicked_function, void *cancel_clicked_function)
+void make_save_dialog(char *title, char *filename, void *ok_clicked_function, void *cancel_clicked_function, char *message)
 {
 	GtkWidget *fs;
+	GtkWidget *label;
 	
 	if((fs = gtk_file_selection_new(title)) != NULL)
 	{
@@ -488,6 +553,14 @@ void make_save_dialog(char *title, char *filename, void *ok_clicked_function, vo
 			gtk_signal_connect(GTK_OBJECT(GTK_FILE_SELECTION(fs)->cancel_button), "clicked", kill_widget_argument, fs);
 
 		gtk_signal_connect(GTK_OBJECT(fs), "delete_event", gtk_widget_destroy, fs);
+		
+		if(message)
+		{
+			label = gtk_label_new(message);
+
+			gtk_box_pack_end(GTK_BOX(GTK_FILE_SELECTION(fs)->main_vbox), label, FALSE, FALSE, 5);
+		}
+		
 		gtk_widget_show_all(fs);
 	}
 }
@@ -500,8 +573,7 @@ static void do_save(GtkWidget *widget, int data)
 		return;
 	}
 	else
-		make_save_dialog("Save As", main_window->filename, (void *)write_config_file, NULL);
-
+		make_save_dialog("Save As", main_window->filename, (void *)write_config_file, NULL, NULL);
 }
 
 void open_file_and_read_config_file(GtkWidget *w, GtkWidget *fs)
@@ -529,42 +601,146 @@ static void do_open(GtkWidget *widget, gpointer data)
 	save_changes(FALSE); // don't quit	
 }
 
-int start_ipv4_agent(char *hostname)
+static void do_zoom(GtkWidget *w, gpointer data)
 {
-	if ((primary_agent = event_request_agent(AGENT_TYPE_IPV4, hostname, 1)))
+	switch((int)data)
 	{
-		agent_ip = makestring(hostname);
-		printf("Connected to ipv4 agent.\n\n");
-		register_gui_handlers();
-		return(1);
-	} 
-	else 
-	{
-		clog(LOG_ERROR, "Unable to connect to local agent\n");
-		return(0);
+		case 0:
+			net_page_zoom_normal(get_current_net_page());
+			break;
+
+		case 1:
+			net_page_zoom_in(get_current_net_page());
+			break;
+
+		case 2:
+			net_page_zoom_out(get_current_net_page());
+			break;
+			
+		default:
+			printf("do_map(): there should not be a default\n");
 	}
 }
 
-void connect_to_ipv4_agent(GtkWidget *w, char *host)
+int close_app(GtkWidget *widget, gpointer data)
 {
-	if (strlen(host))
+   gtk_main_quit();
+   return TRUE;
+}
+
+// http://cvs.gnome.org/lxr/source/imlib/
+// this place had some info about saving
+int export_timeout_callback(void *data)
+{
+	net_page *np = get_current_net_page();
+	GdkImlibImage *image;
+	GtkAdjustment *vadj, *hadj;
+	char *name = data;
+	
+	if(np && np->canvas && name && name[0])
 	{
-		if((main_window->connected = start_ipv4_agent(host)))
+		hadj = gtk_scrolled_window_get_hadjustment(GTK_SCROLLED_WINDOW(np->scrolled_window));
+		vadj = gtk_scrolled_window_get_vadjustment(GTK_SCROLLED_WINDOW(np->scrolled_window));
+
+		DEBUG(
+			printf("H adj\n"
+			       "lower          %f\n"
+			       "upper          %f\n"
+			       "value          %f\n"
+			       "step_increment %f\n"
+			       "page_increment %f\n"
+			       "page_size      %f\n",
+			       hadj->lower,
+			       hadj->upper,
+			       hadj->value,
+			       hadj->step_increment,
+			       hadj->page_increment,
+			       hadj->page_size);
+			printf("H adj\n"
+			       "lower          %f\n"
+			       "upper          %f\n"
+			       "value          %f\n"
+			       "step_increment %f\n"
+			       "page_increment %f\n"
+			       "page_size      %f\n",
+			       vadj->lower,
+			       vadj->upper,
+			       vadj->value,
+			       vadj->step_increment,
+			       vadj->page_increment,
+			       vadj->page_size);
+		);
+		gnome_canvas_update_now(GNOME_CANVAS(np->canvas));
+		gnome_canvas_request_redraw(GNOME_CANVAS(np->canvas), 
+		                            0, 0,
+		                            hadj->page_size, 
+		                            vadj->page_size);
+		while(gtk_events_pending())
+			gtk_main_iteration();                            
+		
+		image = gdk_imlib_create_image_from_drawable(np->canvas->window, 
+		                                             NULL,
+		                                             0, 0, 
+		                                             hadj->page_size, 
+		                                             vadj->page_size);
+
+		if(gdk_imlib_save_image(image, name, NULL))
 		{
-			if(main_window->agent_ip)
-				free(main_window->agent_ip);
-				
-			main_window->agent_ip = makestring(host);
+			printf("file saved '%s'\n", name);
+		}
+		else
+		{
+			printf("file not saved '%s'\n", name);
 		}
 	}
+	if(name)
+		free(name);
 
-	if(!main_window->connected)
-	{
-		make_gnome_ok_dialog("Could not connect to agent", "I could not connect to the Specified IP agent");
-	}
+	return(0);
 }
 
-void make_main_window(cheops_window *w, int connected)
+void export_callback(GtkWidget *w, GtkWidget *fs)
+{
+	char *name = makestring(gtk_file_selection_get_filename(GTK_FILE_SELECTION(fs)));
+	net_page *np = get_current_net_page();
+	
+	gtk_widget_destroy(fs);
+	gnome_canvas_update_now(GNOME_CANVAS(np->canvas));
+	while(gtk_events_pending())
+		gtk_main_iteration();
+
+	cheops_sched_add(100, export_timeout_callback, name);
+}
+
+static void do_export(GtkWidget *w, gpointer data)
+{
+	char *name;
+	net_page *np = get_current_net_page();
+	
+	if(np->name && strlen(np->name) > 0)
+	{
+		name = malloc(strlen(np->name) + 1 + 4);
+		sprintf(name, "%s.gif", np->name);
+	}
+	else
+	{
+		name = malloc(strlen("map.gif") + 1);
+		strcpy(name, "map.gif");
+	}
+	make_save_dialog("Export Viewspace to a graphic file (saved by extension)", 
+	                 name, 
+	                 (void *)export_callback, 
+	                 NULL,
+	                 "The graphic type will be selected by the extention\n"
+	                 "The applicable types are:"
+	                 "gif, png, jpg, xpm, bmp");
+
+	free(name);
+}
+
+
+
+void make_main_window(cheops_window *w)
 {
 	static GtkItemFactoryEntry menu_items[] = 
 	{
@@ -574,6 +750,8 @@ void make_main_window(cheops_window *w, int connected)
 		{ "/File/_Save",                        "<control>S", do_save,               0 },
 		{ "/File/Save _As",                     "",           do_save,               1 },
 		{ "/File/sep1",                         NULL,         NULL,                  0, "<Separator>" },
+		{ "/File/Export Picture...",            "",           do_export,             0 },
+		{ "/File/sep1",                         NULL,         NULL,                  0, "<Separator>" },
 		{ "/File/_Quit",                        "<control>Q", do_quit,               0 },
 		{ "/_Edit",                             NULL,         NULL,                  0, "<Branch>" },
 		{ "/Edit/tearoff1",                     NULL,         NULL,                  0, "<Tearoff>" },
@@ -584,6 +762,8 @@ void make_main_window(cheops_window *w, int connected)
 		{ "/_Viewspace",                        NULL,         NULL,                  0, "<Branch>" },
 		{ "/Viewspace/tearoff1",                NULL,         NULL,                  0, "<Tearoff>" },
 		{ "/Viewspace/_New...",                 "<control>N", do_page,               0 },
+		{ "/Viewspace/Rename",                  NULL,         do_page,               2 },
+		{ "/Viewspace/sep0",                    NULL,         NULL,                  0, "<Separator>" },
 		{ "/Viewspace/_Remove",                 NULL,         do_page,               1 },
 		{ "/Viewspace/sep1",                    NULL,         NULL,                  0, "<Separator>" },
 		{ "/Viewspace/Add _Host...",            "<control>H", do_network,            0 },
@@ -594,6 +774,11 @@ void make_main_window(cheops_window *w, int connected)
 		{ "/Viewspace/Map/_Map Everything",     "<control>M", do_map,                0 },
 		{ "/Viewspace/Map/ReMap Everything",    NULL,         do_map,                1 },
 		{ "/Viewspace/Map/UnMap Everything",    "<control>U", do_unmap,              0 },
+		{ "/Viewspace/sep2",                    NULL,         NULL,                  0, "<Separator>" },
+		{ "/Viewspace/Zoom/",                   NULL,         NULL,                  0 },
+		{ "/Viewspace/Zoom/Normal",             NULL,         do_zoom,               0 },
+		{ "/Viewspace/Zoom/In",                 NULL,         do_zoom,               1 },
+		{ "/Viewspace/Zoom/Out",                NULL,         do_zoom,               2 },
 		{ "/_Help",                             NULL,         NULL,                  0, "<LastBranch>" },
 		{ "/Help/About...",                     NULL,         do_about,              0 },
 		{ "/Help/License...",                   NULL,         do_license,            0 },
@@ -602,14 +787,9 @@ void make_main_window(cheops_window *w, int connected)
 	int nmenu_items = sizeof(menu_items)/sizeof(menu_items[0]);
 	char buf[256];
 	FILE *fp;
-	GtkWidget *dialog;
-	GtkWidget *hbox;
-	GtkWidget *label;
-	GtkWidget *entry1;
-
+			
 	w->net_pages = NULL;
 	w->filename = NULL;
-	w->connected = connected;
 	
 	w->window = gnome_app_new(CHEOPS_TITLE, CHEOPS_TITLE);
 	gtk_widget_set_usize(w->window, CHEOPS_MIN_WIDTH, CHEOPS_MIN_HEIGHT);
@@ -638,61 +818,15 @@ void make_main_window(cheops_window *w, int connected)
 	gtk_widget_show(w->menu);
 	gtk_widget_show(w->window);
 
-//	while(gtk_events_pending())
-//		gtk_main_iteration();
-		
-	main_window->agent_ip = agent_ip;
-	
 	init_net_page_stuff();
-		
-/* if we did not connect, that means that we could not connect to a local agent, so try a ipv4 one */
-	while(!w->connected)
-	{
-		/* do something to give them a prompt to loginto a netaork agent */
-		dialog = gnome_dialog_new("Connect to Agent",
-						  GNOME_STOCK_BUTTON_OK,
-						  GNOME_STOCK_BUTTON_CANCEL,
-						  NULL);
-		gnome_dialog_close_hides(GNOME_DIALOG(dialog), FALSE);
-		
-		entry1 = gnome_entry_new("");
-		label = gtk_label_new("Agent hostname:");
-		hbox = gtk_hbox_new(FALSE, 5);
-		gnome_dialog_set_parent(GNOME_DIALOG(dialog), GTK_WINDOW(main_window->window));
-		
-		gtk_box_pack_start( GTK_BOX(hbox), label, FALSE, FALSE, 5);
-		gtk_box_pack_start( GTK_BOX(hbox), entry1, FALSE, FALSE, 5);
-		gtk_box_pack_start( GTK_BOX( GNOME_DIALOG(dialog)->vbox ), hbox, TRUE, TRUE, 0);
 
-		gtk_widget_grab_focus(GTK_COMBO(entry1)->entry);
-
-		gtk_signal_connect(GTK_OBJECT(GTK_COMBO(entry1)->entry),  "activate", GTK_SIGNAL_FUNC(click_ok_on_gnome_dialog), dialog);
-			
-		gtk_widget_show( hbox );
-		gtk_widget_show( label );
-		gtk_widget_show( entry1 );
-
-		switch( gnome_dialog_run(GNOME_DIALOG(dialog)) )
-		{
-			case 0:
-				connect_to_ipv4_agent(NULL, gtk_entry_get_text(GTK_ENTRY(gnome_entry_gtk_entry(GNOME_ENTRY(entry1)))));
-				gnome_dialog_close(GNOME_DIALOG(dialog));
-				break;
-			case 1:	
-				gnome_dialog_close(GNOME_DIALOG(dialog));
-				exit(0);
-			default:
-				break;
-		}
-	}
-	
 /* lets open the default ~/.gnome/cheops-ng.map file to read the config */
 	sprintf(buf,"%s/%s",getenv("HOME"), CHEOPS_DEFAULT_FILENAME_NOT_HOME);
 
 	if(NULL == (fp = fopen(buf,"r")))
 	{
 		w->filename = NULL;
-		make_viewspace("My Network Stuff");
+		make_viewspace("My Network Stuff", NULL, 0);
 	}
 	else
 	{
@@ -700,14 +834,11 @@ void make_main_window(cheops_window *w, int connected)
 		w->filename = makestring(buf);
 	}
 	read_config_file(NULL,NULL);
-
+	apply_all_settings();
 }
 
 int main(int argc, char *argv[])
 {
-	int connected=0;
-	char *ip;
-
 	gnome_init(CHEOPS_TITLE, CHEOPS_VERSION ,argc,argv);
 
 	if (!getenv("HOME"))
@@ -715,31 +846,14 @@ int main(int argc, char *argv[])
 		fprintf(stderr, "No home directory!\n");
 		return(1);
 	}
-	else
-	{
-		if((primary_agent = event_request_agent(AGENT_TYPE_LOCAL, NULL, 1)))
-		{	
-			printf("Connected to local agent.\n\n");
-			register_gui_handlers();
-			connected = 1;
-		}
-		else 
-		{
-			connected = 0;
 
-			if((ip = config_get_agent_ip_address()) != NULL)
-			{
-				connected = start_ipv4_agent(ip);
-				free(ip);
-			}
-		}
-	}
+	register_gui_handlers();
 
 	if( (main_window = NEW(cheops_window)) == NULL)
 	{
 		quit();
 	}
-	make_main_window(main_window, connected);
+	make_main_window(main_window);
 
 	gnome_dns_init(2); //use 2 dns servers
 	gtk_main();
