@@ -28,9 +28,11 @@
 #include <sys/un.h>
 #include <sys/stat.h>
 #include <netinet/in.h>
+
 #ifdef FREEBSD
-#include <netinet/in_systm.h>
+	#include <netinet/in_systm.h>
 #endif
+
 #include <netinet/ip.h>
 #include <netinet/tcp.h>
 #include <arpa/inet.h>
@@ -44,12 +46,17 @@
 #include <pthread.h>
 #include "logger.h"
 
+#ifdef HAS_SSL
+	#include <openssl/ssl.h>
+	#include <openssl/err.h>
+	#include <openssl/rand.h>
+#endif
 
 #ifdef COMPILING_GUI
-#include <gtk/gtk.h>
-#include "gui-io.h"
+	#include <gtk/gtk.h>
+	#include "gui-io.h"
 #else
-#include "io.h"
+	#include "io.h"
 #endif
 
 #include "misc.h"
@@ -57,12 +64,20 @@
 #include "cerror.h"
 #include "ip_utils.h"
 
+#include <shadow.h>
+#include <pwd.h>
+
+char *crypt(const char *key, const char *salt);
+
+
 /* Maximum length of local/unix socket */
 #define MAX_SUN_LEN 108
 
 /* Number of connections to let wait */
 #define AGENT_BACKLOG 10
 
+//#define DEBUG_EVENT
+
 #ifdef DEBUG_EVENT
 	#define DEBUG(a) a
 #else
@@ -70,9 +85,12 @@
 #endif
 
 #ifdef FREEBSD
-#define PF_FILE PF_LOCAL
+	#define PF_FILE PF_LOCAL
 #endif
 
+#define MY_CERT "/etc/ssl/cheops-agent.crt"
+#define MY_PRIVATE_KEY MY_CERT
+
 /* How many buckets for the event hash? */
 static cheops_event_handler handlers[MAX_EVENT];
 
@@ -82,12 +100,88 @@ static agent *our_local_ipv4_agent = NULL;
 
 static agent *agents=NULL;
 
+int authenticate_connecting_clients = 0;
+
+#ifdef HAS_SSL
+	int initalized_ssl = 0;
+	int connect_with_ssl = 0;
+	SSL_CTX *ssl_ctx;
+#endif
 
 static char unix_connect_string[] 	= "handle_unix_connect";
 static char local_string[] 		= "local_agent";
 static char ipv4_connect_string[] 	= "handle_ipv4_connect";
 static char ipv4_string[] 		= "ipv4_agent";
 
+int handle_auth_request(event_hdr *h, event *e, agent *a);
+int send_auth_request_reply(agent *a, int authenticated);
+
+void agent_authenticate_clients(agent *a, int enable)
+{
+	if(enable)
+	{
+		if(a)
+			a->flags |= AGENT_FLAGS_AUTHENTICATE_CLIENTS;
+	}
+	else
+	{
+		if(a)
+			a->flags &= ~AGENT_FLAGS_AUTHENTICATE_CLIENTS;
+	}
+	
+}
+int agent_authenticating_clients(agent *a)
+{
+	if(a && a->flags & AGENT_FLAGS_AUTHENTICATE_CLIENTS)
+		return(1);
+	
+	return(0);  
+}
+                
+                
+void authenticate_clients(int enable)
+{
+	authenticate_connecting_clients = (enable ? 1 : 0);
+}
+
+int authenticating_clients(void)
+{
+	return(authenticate_connecting_clients);
+}
+
+#ifdef HAS_SSL
+void agent_use_ssl(agent *a, int enable)
+{
+	if(enable)
+	{
+		if(a)
+			a->flags |= AGENT_FLAGS_USING_SSL;
+	}
+	else
+	{
+		if(a)
+			a->flags &= ~AGENT_FLAGS_USING_SSL;
+	}
+}
+
+void use_ssl(int enable)
+{
+	connect_with_ssl = (enable ? 1 : 0);
+}
+
+int using_ssl(void)
+{
+	return(connect_with_ssl);
+}
+
+int agent_using_ssl(agent *a)
+{
+	if(a && a->flags & AGENT_FLAGS_USING_SSL)
+		return(1);
+	
+	return(0);	
+}
+#endif /* HAS_SSL */
 
 static agent *new_agent()
 {
@@ -96,6 +190,7 @@ static agent *new_agent()
 	{
 		a->write_ptr = a->buffer;
 		a->expected_length = -1;
+		a->flags = 0;
 	}
 	return(a);
 }
@@ -181,8 +276,6 @@ static int handle_agent(int *id, int fd, short events, void *data)
 		{
 			DEBUG(clog(LOG_WARNING,
 				"Closing connection on read error\n"));
-			close(fd);
-			a->id = 0;
 			event_destroy_agent(a);
 			return 0;
 		}
@@ -190,8 +283,6 @@ static int handle_agent(int *id, int fd, short events, void *data)
 		{
 			DEBUG(clog(LOG_WARNING,
 				"Closing connection on NULL read (socket closed)\n"));
-			close(fd);
-			a->id = 0;
 			event_destroy_agent(a);
 			return 0;
 		}
@@ -217,8 +308,6 @@ static int handle_agent(int *id, int fd, short events, void *data)
 		{
 			DEBUG(clog(LOG_WARNING,
 				"Closing connection on read error\n"));
-			close(fd);
-			a->id = 0;
 			event_destroy_agent(a);
 			return 0;
 		}
@@ -226,8 +315,6 @@ static int handle_agent(int *id, int fd, short events, void *data)
 		{
 			DEBUG(clog(LOG_WARNING,
 				"Closing connection on NULL read (socket closed)\n"));
-			close(fd);
-			a->id = 0;
 			event_destroy_agent(a);
 			return 0;
 		}
@@ -256,18 +343,50 @@ again:
 		if (ntohs(h->type) >= MAX_EVENT){
 			clog(LOG_WARNING,
 				"Message type %d is too large\n", h->type);
-		} else if (ntohs(h->hlen) > ntohl(h->len)) {
+		} 
+		else if (ntohs(h->hlen) > ntohl(h->len)) 
+		{
 			clog(LOG_WARNING,
 				"Message header longer than message itself?\n", ntohs(h->type));
-		} else {
-			e = (event *)(a->buffer + ntohs(h->hlen));
-			DEBUG(clog(LOG_DEBUG, "Event received of type %d\n", ntohs(h->type)));
-			if (handlers[ntohs(h->type)]) 
-				handlers[ntohs(h->type)](h, e, a);
-			else {
-				clog(LOG_DEBUG, "No handler for event type %d\n", ntohs(h->type));
-				if (!(htonl(h->flags) & FLAG_IGNORE)) {
-					cheops_error(a, ERROR_UNIMPL, "Unsupported message type %d\n", ntohs(h->type));
+		} 
+		else 
+		{
+			if((a->flags & AGENT_FLAGS_LOGGED_IN) || !authenticating_clients())
+			{
+				e = (event *)(a->buffer + ntohs(h->hlen));
+				DEBUG(clog(LOG_DEBUG, "Event received of type %d\n", ntohs(h->type)));
+				if (handlers[ntohs(h->type)]) 
+				{
+					handlers[ntohs(h->type)](h, e, a);
+				}
+				else 
+				{
+					clog(LOG_DEBUG, "No handler for event type %d\n", ntohs(h->type));
+					if (!(htonl(h->flags) & FLAG_IGNORE)) 
+					{
+						cheops_error(a, ERROR_UNIMPL, "Unsupported message type %d\n", ntohs(h->type));
+					}
+				}
+			}
+			else
+			{
+				/*
+				 * if the user is not logged in then the only thing that we will
+				 * reply with is event error
+				 */
+				e = (event *)(a->buffer + ntohs(h->hlen));
+				printf("not logged in!\n");
+				if( ntohs(h->type) == EVENT_AUTH_REQUEST)
+				{
+					handlers[EVENT_AUTH_REQUEST](h,e,a);
+				}
+				else
+				{
+					clog(LOG_DEBUG, "No handler for event type %d when not logged in\n", ntohs(h->type));
+					if (!(htonl(h->flags) & FLAG_IGNORE)) 
+					{
+						cheops_error(a, ERROR_UNIMPL, "Unsupported message type %d\n", ntohs(h->type));
+					}
 				}
 			}
 		}
@@ -391,6 +510,7 @@ static int make_local_socket(char *c)
 	int s;
 	int flags;
 	int retry=0;
+	int on = 1;
 	make_home_dir();
 	memset(&sun, 0, sizeof(sun));
 	sun.sun_family = PF_FILE;
@@ -421,6 +541,8 @@ tryagain:
 		if (fcntl(s, F_SETFL, (flags | O_NONBLOCK)) < 0)
 			clog(LOG_WARNING, "fcntl(F_SETFL) failed(%s)\n", strerror(errno));
 
+	setsockopt(s, SOL_SOCKET, SO_REUSEADDR, (void *)&on, sizeof(on));
+
 	if (listen(s, AGENT_BACKLOG) < 0) {
 		clog(LOG_ERROR, "listen() failed (%s)\n", strerror(errno));
 		close(s);
@@ -438,6 +560,7 @@ static int handle_ipv4_connect(int *id, int fd, short events, void *data)
 	agent *a;
 	struct sockaddr_in sin;
 	int sinlen;
+	
 	a = (agent *)data;
 	if (!a) {
 		/* Something bad has happened, try not to repeat it */
@@ -464,7 +587,62 @@ static int handle_ipv4_connect(int *id, int fd, short events, void *data)
 			a->id = cheops_io_add(res, handle_agent, CHEOPS_IO_IN, a);
 			a->next = agents;
 			a->more = &ipv4_connect_string[0];
+			a->flags = 0;
+			
+#ifdef HAS_SSL
+			if( using_ssl() ) {
+				int ret;
+				
+				if(!initalized_ssl) {
+					RAND_seed("/dev/random", 50);	
+					SSL_load_error_strings();
+					SSL_library_init();
+
+					if( (ssl_ctx = SSL_CTX_new(SSLv2_server_method())) == NULL) {
+						clog(LOG_ERROR, "Unable to create the ssl ctx\n");
+					}
+					if (SSL_CTX_use_certificate_file(ssl_ctx, MY_CERT, SSL_FILETYPE_PEM) <= 0) {
+						ERR_print_errors_fp(stderr);
+						exit(3);
+					}
+					if (SSL_CTX_use_PrivateKey_file(ssl_ctx, MY_PRIVATE_KEY, SSL_FILETYPE_PEM) <= 0) {
+						ERR_print_errors_fp(stderr);
+						exit(4);
+					}
+					if (!SSL_CTX_check_private_key(ssl_ctx)) {
+						fprintf(stderr,"Private key does not match the certificate public key\n");
+						exit(5);   
+					}
+				}
+				if((a->ssl = SSL_new(ssl_ctx)) == NULL) {
+					clog(LOG_ERROR, "Unable to create the ssl\n");
+				}
+				if(!SSL_set_fd(a->ssl, a->s)) {
+					clog(LOG_ERROR, "Unable to set the SSL_fd\n");
+				}
+				
+				if((ret = SSL_accept(a->ssl)))
+				{
+					clog(LOG_ERROR, "they are in\n");
+				}
+				else
+				{
+					clog(LOG_ERROR, "they are not in\n");
+				}
+			}
+			else
+#endif /* HAS_SSL */
+			{
+				a->ssl = NULL;
+			}
+
 			agents = a;
+
+			if(authenticating_clients())
+			{
+				DEBUG(printf("Sending auth request reply with !authenticated\n"));
+				send_auth_request_reply(a, 0);
+			}
 		}
 	}
 	
@@ -472,8 +650,7 @@ static int handle_ipv4_connect(int *id, int fd, short events, void *data)
 	return 1;
 }
 
-
-static agent *ipv4_agent(char *ip_addr)
+static agent *ipv4_agent(char *ip_addr, int usessl, int is_client)
 {
 	/* We asynchronously request a ipv4 agent here.
 	   To do so, we allocate some random socket and
@@ -484,7 +661,7 @@ static agent *ipv4_agent(char *ip_addr)
 	int optval=1;
 	
 	struct sockaddr_in sin;
-	if(our_local_ipv4_agent)
+	if(our_local_ipv4_agent && !is_client)
 		return(our_local_ipv4_agent);
 
 	s = socket(PF_INET, SOCK_STREAM, 0);
@@ -496,15 +673,14 @@ static agent *ipv4_agent(char *ip_addr)
 		sin.sin_port = htons(2300);
 		sin.sin_addr.s_addr = inet_addr(ip_addr);
 
-		if(setsockopt(s, SOL_SOCKET, TCP_NODELAY, &optval,sizeof(optval)) < 0)
-		{
-// this can not be set if you are not root			clog(LOG_ERROR, "Unable to set TCP_NODELAY (%s)\n",strerror(errno));
-		}		
+		setsockopt(s, SOL_SOCKET, TCP_NODELAY, &optval,sizeof(optval));
+		setsockopt(s, SOL_SOCKET, SO_REUSEADDR, (void *)&optval, sizeof(optval));	
+		
 		if (connect(s, (void *)&sin, sizeof(sin)) < 0)
 		{
 			/* FIXME: We should fork() and exec cheops-agent to start it
 			   if we cannot connect */
-			clog(LOG_ERROR, "Unable to connect to ipv4 agent (%s)\n", strerror(errno));
+			clog(LOG_ERROR, "Unable to connect to ipv4 agent '%s' (%s)\n", ip_addr, strerror(errno));
 			return NULL;
 		}
 
@@ -514,12 +690,48 @@ static agent *ipv4_agent(char *ip_addr)
 			a->type = AGENT_TYPE_IPV4;
 			a->s = s;
 			a->more = &ipv4_string[0];
+
+#ifdef HAS_SSL
+			if( usessl ) 
+			{
+				if(!initalized_ssl) 
+				{
+					RAND_seed("/dev/random", 50);	
+					SSL_load_error_strings();
+					SSL_library_init();
+
+					if( (ssl_ctx = SSL_CTX_new(SSLv2_client_method())) == NULL) 
+					{
+						clog(LOG_ERROR, "Unable to create the ssl ctx\n");
+					}
+				}
+				if((a->ssl = SSL_new(ssl_ctx)) == NULL) 
+				{
+					clog(LOG_ERROR, "Unable to create the ssl\n");
+				}
+				if(!SSL_set_fd(a->ssl, a->s)) 
+				{
+					clog(LOG_ERROR, "Unable to set the SSL_fd\n");
+				}
+				
+				a->id = cheops_io_add(a->s, handle_agent, CHEOPS_IO_IN, a);
+			}
+			else
+#endif /* HAS_SSL */
+			{
+				a->ssl = NULL;
+				a->id = cheops_io_add(s, handle_agent, CHEOPS_IO_IN | CHEOPS_IO_ERR,a);
+			}
+
 			a->next = agents;
-			a->id = cheops_io_add(s, handle_agent, CHEOPS_IO_IN | CHEOPS_IO_ERR,a);
 			agents = a;
 		}
 	}
-	our_local_ipv4_agent = a;
+	if(!is_client)
+		our_local_ipv4_agent = a;
+#ifdef HAS_SSL	
+	agent_use_ssl(a, usessl);
+#endif
 	return a;
 }
 
@@ -553,6 +765,8 @@ static int make_ipv4_socket(struct sockaddr_in *sin)
 	{
 		clog(LOG_ERROR, "Unable to set TCP_NODELAY");
 	}		
+	setsockopt(s, SOL_SOCKET, SO_REUSEADDR, (void *)&optval, sizeof(optval));
+	
 	if((flags = fcntl(s, F_GETFL)) < 0)
 		clog(LOG_ERROR, "fcntl(F_GETFL) failed(%s)\n", strerror(errno));
 	else
@@ -696,16 +910,19 @@ int event_create_agent(int agent_type)
 				if (res > -1)
 				{
 					a = new_agent();
+
 					if (a) {
+						
 						a->type = AGENT_TYPE_IPV4;
 						a->s = res;
-						a->id = cheops_io_add(res, handle_ipv4_connect, CHEOPS_IO_IN, a);
 						a->next = agents;
 						a->more = NULL;
+						a->ssl = NULL;
+						a->id = cheops_io_add(res, handle_ipv4_connect, CHEOPS_IO_IN, a);
 						agents = a;
-						res=0;
+						res = 0;
 					} else {
-						res=-1;
+						res = -1;
 					}
 				}
 			}
@@ -774,7 +991,7 @@ int event_send(agent *a, event_hdr *h)
 	return res;
 }
 
-agent *event_request_agent(int agent_type, void *addr, int is_default)
+agent *event_request_agent(int agent_type, void *addr, int usessl)
 {
 	/* 
 	 * Requests that an agent connection be established 
@@ -806,7 +1023,7 @@ agent *event_request_agent(int agent_type, void *addr, int is_default)
 	case AGENT_TYPE_IPV4:
 		/* Communicate by requesting an agent
 		   connection over IPV4 */
-		a = ipv4_agent((char *)addr);
+		a = ipv4_agent((char *)addr, usessl, 1);
 		break;
 	default:
 		clog(LOG_ERROR, "unknown agent requested: %d\n", agent_type);
@@ -816,6 +1033,12 @@ agent *event_request_agent(int agent_type, void *addr, int is_default)
 
 extern int event_register_handler(short type, cheops_event_handler h, int replace)
 {
+	static int initalized = 0;
+	
+	if(!initalized){
+		initalized = 1;
+		event_register_handler(EVENT_AUTH_REQUEST, handle_auth_request, 1);
+	}
 	if (type < MAX_EVENT) {
 		if (replace || !handlers[type]) 
 			handlers[type] = h;
@@ -827,3 +1050,75 @@ extern int event_register_handler(short type, cheops_event_handler h, int replac
 }
 
 
+int send_auth_request_reply(agent *a, int authenticated)
+{
+	static char ebuf[MAX_EVENT_SIZE];
+	static event_hdr *eh = (event_hdr *)ebuf;
+	static event *ee = (event *)(ebuf + sizeof(event_hdr));
+	
+	eh->len = htonl(sizeof(event_hdr) + sizeof(auth_request_r));
+	eh->hlen = htons(sizeof(event_hdr));
+	eh->type = htons(REPLY_AUTH_REQUEST);
+	eh->flags = 0;
+	ee->auth_request_r.authenticated = htonl(authenticated); // yes is stupid but oh well
+
+	DEBUG(printf("%s() %d\n", __FUNCTION__, authenticated));
+
+	if (event_send(a, eh) < 0)
+	{
+		DEBUG( clog(LOG_WARNING, "Unable to send discover reply\n") );
+		return(0);
+	}
+	return(1);
+	
+}
+
+int handle_auth_request(event_hdr *h, event *e, agent *a)
+{
+	char *user = e->auth_request_e.username;
+	char *pass = e->auth_request_e.password;
+	char *enc;
+	struct passwd *pw;
+	struct spwd   *spw;
+	int authenticated = 0;
+	
+	DEBUG(
+		printf("%s()\n", __FUNCTION__);
+		printf("user '%s'\n", e->auth_request_e.username);
+		printf("pass '%s'\n", e->auth_request_e.password);
+	);
+
+	if( (pw = getpwnam(user)) == NULL)
+	{
+		send_auth_request_reply(a, 0);
+		return(1);
+	}
+
+	if( (spw = getspnam((char *)pw->pw_name)) == NULL)
+	{
+		send_auth_request_reply(a, 0);
+		return(1);
+	}	
+	enc = crypt(pass, (char *)spw->sp_pwdp);
+	DEBUG(
+		printf("crypt = %s\n"
+		       "pwdb  = %s\n",
+		       enc,
+		       (char *)spw->sp_pwdp);
+	)
+	if(0 == strcmp((char *)spw->sp_pwdp, enc))
+	{
+		a->flags |= AGENT_FLAGS_LOGGED_IN;
+		authenticated = 1;
+	}
+	else
+	{
+		authenticated = 0;
+	}
+	send_auth_request_reply(a, authenticated);
+	return(1);
+}
+
+
+
+
