@@ -37,6 +37,7 @@
 #include "ip_utils.h"
 #include "gui-pixmap.h"
 #include "gui-handlers.h"
+#include "cheops-osscan.h"
 
 //#define DEBUG_CONFIG
 
@@ -96,7 +97,7 @@ char *config_get_agent_ip_address(void)
 	}
 	fclose(fp);
 	
-	sprintf(buf,"=%s=/global/agent_ip=NO_IP",buf2);
+	sprintf(buf,"=%s=/viewspace0/agent_ip=NO_IP",buf2);
 	
 	ip = gnome_config_get_string_with_default(buf,&def);
 	if(def)
@@ -108,6 +109,35 @@ char *config_get_agent_ip_address(void)
 	return(ip);	
 }
 
+int *config_get_agent_usessl(void)
+{
+	int def;
+	int *usessl;
+	char buf[CONFIG_BUFFER_SIZE];
+	char buf2[CONFIG_BUFFER_SIZE];
+	FILE *fp;
+	
+	sprintf(buf2,"%s/%s",getenv("HOME"), CHEOPS_DEFAULT_FILENAME_NOT_HOME);
+	
+	if( (fp = fopen(buf2, "rt")) == NULL)
+	{
+		return(NULL);
+	}
+	fclose(fp);
+	
+	sprintf(buf,"=%s=/viewspace0/agent_usessl=0",buf2);
+	
+	usessl = malloc(sizeof(int));
+	*usessl = gnome_config_get_int_with_default(buf,&def);
+	if(def)
+	{
+		free(usessl);
+		usessl = NULL;
+	}
+	
+	return(usessl);	
+}
+
 void read_config_file(GtkWidget *w, GtkWidget *fs)
 {
 	char buf[CONFIG_BUFFER_SIZE], buf2[CONFIG_BUFFER_SIZE], base[CONFIG_BUFFER_SIZE], *filename;
@@ -127,6 +157,8 @@ void read_config_file(GtkWidget *w, GtkWidget *fs)
 	int service_protocol;
 	char *service_command;
 	os_port_entry *current = NULL;
+	char *agent_ip;	
+	double ppu;
 	
 // default the service list and pixmaps list so the use without a config file can use these
 	os_pixmap_list_default();
@@ -205,16 +237,33 @@ void read_config_file(GtkWidget *w, GtkWidget *fs)
 	sprintf(buf,"=%s=/global/options_os_scan_tcp_scan=%s",filename, options_os_scan_tcp_scan);
 	temp_string = gnome_config_get_string_with_default(buf,&def);
 	if(def)
-		strncpy(options_os_scan_tcp_scan, TCP_SCAN_DEFAULT, sizeof(options_os_scan_ports));
+		strncpy(options_os_scan_tcp_scan, TCP_SCAN_DEFAULT, sizeof(options_os_scan_tcp_scan));
 	else
 		strncpy(options_os_scan_tcp_scan, temp_string, sizeof(options_os_scan_tcp_scan));
 	free(temp_string);
 	apply_the_setting(OPTIONS_OS_SCAN_TCP_SCAN);
 
+	sprintf(buf,"=%s=/global/options_os_scan_timing=%s",filename, options_os_scan_timing);
+	temp_string = gnome_config_get_string_with_default(buf,&def);
+	if(def)
+		strncpy(options_os_scan_timing, TIMING_SCAN_DEFAULT, sizeof(options_os_scan_timing));
+	else
+		strncpy(options_os_scan_timing, temp_string, sizeof(options_os_scan_timing));
+	free(temp_string);
+	apply_the_setting(OPTIONS_OS_SCAN_TIMING);
+
 	sprintf(buf,"=%s=/global/options_os_scan_udp_scan=%d",filename, options_os_scan_udp_scan);
 	options_os_scan_udp_scan = gnome_config_get_int_with_default(buf,&def);
 	apply_the_setting(OPTIONS_OS_SCAN_UDP_SCAN);
 
+	sprintf(buf,"=%s=/global/options_os_scan_rpc_scan=%d",filename, options_os_scan_rpc_scan);
+	options_os_scan_rpc_scan = gnome_config_get_int_with_default(buf,&def);
+	apply_the_setting(OPTIONS_OS_SCAN_RPC_SCAN);
+
+	sprintf(buf,"=%s=/global/options_os_scan_identd_scan=%d",filename, options_os_scan_identd_scan);
+	options_os_scan_identd_scan = gnome_config_get_int_with_default(buf,&def);
+	apply_the_setting(OPTIONS_OS_SCAN_IDENTD_SCAN);
+
 	sprintf(buf,"=%s=/global/options_os_scan_fastscan=%d",filename, options_os_scan_fastscan);
 	options_os_scan_fastscan = gnome_config_get_int_with_default(buf,&def);
 	apply_the_setting(OPTIONS_OS_SCAN_FASTSCAN);
@@ -325,18 +374,43 @@ void read_config_file(GtkWidget *w, GtkWidget *fs)
 	
 	for(i=0;;i++)
 	{
+		int usessl = 0;
+		
 		sprintf(base,"=%s=/viewspace%d",filename,i);
 		
 		sprintf(buf,"%s%s",base, "/name=NOTHING");
 		name = gnome_config_get_string_with_default(buf, &def);
-				
 		if(def) /* if it uses the default then there are no more viewspaces in the config file */
 		{
 			free(name);
 			break;
 		}
-		np = make_viewspace(name);
+
+		sprintf(buf,"%s%s",base, "/agent_ip=NOTHING");
+		agent_ip = gnome_config_get_string_with_default(buf, &def);
+		if(def) 
+		{
+			free(agent_ip);
+			agent_ip = NULL;
+		}
 		
+		sprintf(buf,"%s%s",base, "/usessl=NOTHING");
+		usessl = gnome_config_get_int_with_default(buf, &def);
+		if(def) 
+		{
+			usessl = 0;
+		}
+		np = make_viewspace(name, agent_ip, usessl);
+		if(name)
+			free(name);
+		if(agent_ip)
+			free(agent_ip);
+		
+		sprintf(buf,"%s%s",base, "/pixels_per_unit=NOTHING");
+		ppu = (double)gnome_config_get_float_with_default(buf, &def);
+		if(!def) 
+			net_page_set_pixels_per_unit(np, ppu);
+
 		while(gtk_events_pending())		/* have to let the notebook add the page */
 			gtk_main_iteration();
                                                        			
@@ -414,6 +488,7 @@ void read_config_file(GtkWidget *w, GtkWidget *fs)
 			else
 			{
 				os_data = malloc(sizeof(*os_data));
+				memset(os_data, 0, sizeof(*os_data));
 				os_data->ports = NULL;
 				os_data->os = os_name;
 			}
@@ -423,6 +498,12 @@ void read_config_file(GtkWidget *w, GtkWidget *fs)
 				char port_buffer[1024];
 				int port;
 				int protocol;
+				int state;
+				int rpcnum;
+				int rpclowver;
+				int rpchighver;
+				int proto;
+				char *owner;
 				char *name;
 				char *version;
 				os_port_entry *osp;
@@ -441,6 +522,29 @@ void read_config_file(GtkWidget *w, GtkWidget *fs)
 				if(def)
 					break;
 
+				snprintf(port_buffer, sizeof(port_buffer), "_port_%d_state=%d", port_count, PORT_STATE_OPEN);
+				add_name_to_buf(buf, buf2, port_buffer);
+				state = gnome_config_get_int_with_default(buf2,&def);
+
+				snprintf(port_buffer, sizeof(port_buffer), "_port_%d_proto=0", port_count);
+				add_name_to_buf(buf, buf2, port_buffer);
+				proto = gnome_config_get_int_with_default(buf2,&def);
+				
+				if(def)
+					state = PORT_PROTO_NONE;
+
+				snprintf(port_buffer, sizeof(port_buffer), "_port_%d_rpcnum=0", port_count);
+				add_name_to_buf(buf, buf2, port_buffer);
+				rpcnum = gnome_config_get_int_with_default(buf2,&def);
+
+				snprintf(port_buffer, sizeof(port_buffer), "_port_%d_rpclowver=0", port_count);
+				add_name_to_buf(buf, buf2, port_buffer);
+				rpclowver = gnome_config_get_int_with_default(buf2,&def);
+
+				snprintf(port_buffer, sizeof(port_buffer), "_port_%d_rpchighver=0", port_count);
+				add_name_to_buf(buf, buf2, port_buffer);
+				rpchighver = gnome_config_get_int_with_default(buf2,&def);
+
 				snprintf(port_buffer, sizeof(port_buffer), "_port_%d_name=BLAH", port_count);
 				add_name_to_buf(buf, buf2, port_buffer);
 				name = gnome_config_get_string_with_default(buf2,&def);
@@ -461,16 +565,34 @@ void read_config_file(GtkWidget *w, GtkWidget *fs)
 					version = NULL;
 				}
 				
+				snprintf(port_buffer, sizeof(port_buffer), "_port_%d_owner=BLAH", port_count);
+				add_name_to_buf(buf, buf2, port_buffer);
+				owner = gnome_config_get_string_with_default(buf2,&def);
+				
+				if(def)
+				{
+					free(owner);
+					owner = NULL;
+				}
+
 				osp = malloc(sizeof(*osp));
+				memset(osp, 0, sizeof(*osp));
 				osp->name = name;
 				osp->version = version;
 				osp->port = port;
 				osp->protocol = protocol;
 				osp->next = NULL;
+				osp->state = state;
+				osp->proto = proto;
+				osp->rpcnum = rpcnum;
+				osp->rpclowver = rpclowver;
+				osp->rpchighver = rpchighver;
+				osp->owner = owner;
 				
 				if(os_data == NULL)
 				{
 					os_data = malloc(sizeof(*os_data));
+					memset(os_data, 0, sizeof(*os_data));
 					os_data->os = NULL;
 					os_data->ports = NULL;
 				}
@@ -488,7 +610,7 @@ void read_config_file(GtkWidget *w, GtkWidget *fs)
 				}
 			}
 
-			po = add_discovered_node(primary_agent, np, str2ip(po_ip), po_name, os_data, FALSE); 
+			po = add_discovered_node(np->agent, np, str2ip(po_ip), po_name, os_data, FALSE); 
 			free(po_ip);
 			
 			if(!po)
@@ -665,16 +787,23 @@ void write_config_file(GtkWidget *w, GtkWidget *fs)
 	
 	/* clear out the destination file if it is there */
 	sprintf(base,"=%s=",filename);
+	/* 
+	 * for some reason the clean_file does not get rid of all data
+	 * so i have to do this stupid loop
+	 */
+	i = 0;
+	while(1)
+	{
+		sprintf(buf, "=%s=/viewspace%d", filename, i);
+		if(gnome_config_has_section(buf))
+			gnome_config_clean_section(buf);
+		else
+			break;
+		i++;
+	}
 	gnome_config_clean_file(base);
 	gnome_config_sync();
 	
-	/* lets write something in it */
-	if(main_window->agent_ip)
-	{
-		sprintf(buf,"=%s=/global/agent_ip",filename);
-		gnome_config_set_string(buf,main_window->agent_ip);
-	}
-
 	sprintf(buf,"=%s=/global/options_rescan_at_startup",filename);
 	gnome_config_set_int(buf,options_rescan_at_startup);
 
@@ -711,12 +840,21 @@ void write_config_file(GtkWidget *w, GtkWidget *fs)
 	sprintf(buf,"=%s=/global/options_os_scan_tcp_scan",filename);
 	gnome_config_set_string(buf,options_os_scan_tcp_scan);
 
+	sprintf(buf,"=%s=/global/options_os_scan_timing",filename);
+	gnome_config_set_string(buf,options_os_scan_timing);
+
 	sprintf(buf,"=%s=/global/options_os_scan_fastscan",filename);
 	gnome_config_set_int(buf,options_os_scan_fastscan);
 
 	sprintf(buf,"=%s=/global/options_os_scan_udp_scan",filename);
 	gnome_config_set_int(buf,options_os_scan_udp_scan);
 
+	sprintf(buf,"=%s=/global/options_os_scan_identd_scan",filename);
+	gnome_config_set_int(buf,options_os_scan_identd_scan);
+
+	sprintf(buf,"=%s=/global/options_os_scan_rpc_scan",filename);
+	gnome_config_set_int(buf,options_os_scan_rpc_scan);
+
 	sprintf(buf,"=%s=/global/options_os_scan_osscan",filename);
 	gnome_config_set_int(buf,options_os_scan_osscan);
 
@@ -774,6 +912,12 @@ void write_config_file(GtkWidget *w, GtkWidget *fs)
 		sprintf(buf,"%s%s",base, "/name");
 		gnome_config_set_string(buf,np->name);
 		
+		sprintf(buf,"%s%s",base, "/agent_ip");
+		gnome_config_set_string(buf,np->agent_ip);
+		
+		sprintf(buf,"%s%s",base, "/usessl");
+		gnome_config_set_int(buf,(np->agent->flags & AGENT_FLAGS_USING_SSL?1:0));
+		
 		/* add the reference to the page_objects in the netpage sections */
 		strcat(base,"/page_object_");
 
@@ -822,6 +966,26 @@ void write_config_file(GtkWidget *w, GtkWidget *fs)
 						add_name_to_buf(buf, buf2, port_buffer);
 						gnome_config_set_int(buf2,osp->protocol);
 
+						snprintf(port_buffer, sizeof(port_buffer), "_port_%d_state", port_count);
+						add_name_to_buf(buf, buf2, port_buffer);
+						gnome_config_set_int(buf2,osp->state);
+
+						snprintf(port_buffer, sizeof(port_buffer), "_port_%d_proto", port_count);
+						add_name_to_buf(buf, buf2, port_buffer);
+						gnome_config_set_int(buf2,osp->proto);
+
+						snprintf(port_buffer, sizeof(port_buffer), "_port_%d_rpcnum", port_count);
+						add_name_to_buf(buf, buf2, port_buffer);
+						gnome_config_set_int(buf2,osp->rpcnum);
+
+						snprintf(port_buffer, sizeof(port_buffer), "_port_%d_rpclowver", port_count);
+						add_name_to_buf(buf, buf2, port_buffer);
+						gnome_config_set_int(buf2,osp->rpclowver);
+
+						snprintf(port_buffer, sizeof(port_buffer), "_port_%d_rpchighver", port_count);
+						add_name_to_buf(buf, buf2, port_buffer);
+						gnome_config_set_int(buf2,osp->rpchighver);
+
 						if(osp->name)
 						{
 							snprintf(port_buffer, sizeof(port_buffer), "_port_%d_name", port_count);
@@ -835,6 +999,13 @@ void write_config_file(GtkWidget *w, GtkWidget *fs)
 							add_name_to_buf(buf, buf2, port_buffer);
 							gnome_config_set_string(buf2,osp->version);
 						}
+
+						if(osp->owner)
+						{
+							snprintf(port_buffer, sizeof(port_buffer), "_port_%d_owner", port_count);
+							add_name_to_buf(buf, buf2, port_buffer);
+							gnome_config_set_string(buf2,osp->owner);
+						}
 					}
 				}
 			}
