@@ -59,7 +59,10 @@ static GtkWidget *os_scan_ports;
 static GtkWidget *os_scan_ports_label;
 static GtkWidget *os_scan_udp_scan;
 static GtkWidget *os_scan_tcp_scan;
+static GtkWidget *os_scan_timing;
 static GtkWidget *os_scan_dont_ping;
+static GtkWidget *os_scan_rpc_scan;
+static GtkWidget *os_scan_identd_scan;
 static GtkWidget *os_scan_fastscan;
 static GtkWidget *os_scan_osscan;         // this is the os scan? y/n
 static GtkWidget *os_scan_scan_specified_ports;
@@ -91,9 +94,12 @@ int options_use_ip_for_merged_ports = 0;
 unsigned int options_os_scan_flags = 0;
 char options_os_scan_ports[OPTIONS_OS_SCAN_PORTS_SIZE];
 char options_os_scan_tcp_scan[OPTIONS_OS_SCAN_TCP_SCAN_SIZE] = TCP_SCAN_DEFAULT;
-int options_os_scan_udp_scan = 1;
+char options_os_scan_timing[OPTIONS_OS_SCAN_TIMING_SIZE] = TIMING_SCAN_DEFAULT;
+int options_os_scan_udp_scan = 0;
 int options_os_scan_fastscan = 1;
 int options_os_scan_osscan = 1;
+int options_os_scan_rpc_scan = 1;
+int options_os_scan_identd_scan = 1;
 int options_os_scan_dont_ping = 1;
 int options_os_scan_scan_specified_ports = 0;
 
@@ -135,6 +141,7 @@ void apply_the_setting(int flags);
 
 void apply_the_setting(int flags)
 {
+	net_page *np;
 	switch(flags)
 	{
 		case OPTIONS_DISCOVER_RETRIES:
@@ -144,8 +151,9 @@ void apply_the_setting(int flags)
 			eh->flags = 0;
 			ee->set_settings_e.discover_retries = options_discover_retries;
 			ee->set_settings_e.flags = flags;
-			if (event_send(primary_agent, eh) < 0) 
-				clog(LOG_WARNING, "Unable to send set settings event\n");
+			for(np = main_window->net_pages; np; np = np->next)
+				if (event_send(np->agent, eh) < 0) 
+					clog(LOG_WARNING, "Unable to send set settings event\n");
 			break;
 			
 		case OPTIONS_USE_IP_FOR_LABEL:
@@ -194,11 +202,7 @@ void apply_the_setting(int flags)
 				options_os_scan_flags &= ~OS_SCAN_OPTION_DONT_PING;
 			break;
 		case OPTIONS_OS_SCAN_TCP_SCAN:
-			options_os_scan_flags &= ~(OS_SCAN_OPTION_TCP_CONNECT_SCAN | 
-			                           OS_SCAN_OPTION_TCP_SYN_SCAN |
-			                           OS_SCAN_OPTION_STEALTH_FIN |
-			                           OS_SCAN_OPTION_STEALTH_XMAS |
-			                           OS_SCAN_OPTION_STEALTH_NULL);
+			options_os_scan_flags &= ~(OS_SCAN_OPTION_SCAN_MASK);
 			if(0 == strcmp(options_os_scan_tcp_scan, TCP_SCAN_0))
 				options_os_scan_flags |= OS_SCAN_OPTION_TCP_CONNECT_SCAN;
 			else if(0 == strcmp(options_os_scan_tcp_scan, TCP_SCAN_1))
@@ -211,10 +215,31 @@ void apply_the_setting(int flags)
 				options_os_scan_flags |= OS_SCAN_OPTION_STEALTH_NULL;
 			else
 			{
-				printf("you should not see this message again!\n");
+				printf(__FUNCTION__"(): you should not see this message!\n");
 				options_os_scan_flags |= OS_SCAN_OPTION_TCP_CONNECT_SCAN;
 			}
 			
+			break;
+		case OPTIONS_OS_SCAN_TIMING:
+			options_os_scan_flags &= ~(OS_SCAN_OPTION_TIMIMG_MASK);
+			if(0 == strcmp(options_os_scan_timing,      TIMING_SCAN_NORMAL))
+				options_os_scan_flags |= OS_SCAN_OPTION_TIMIMG_NORMAL;
+			else if(0 == strcmp(options_os_scan_timing, TIMING_SCAN_PARANOID))
+				options_os_scan_flags |= OS_SCAN_OPTION_TIMIMG_PARANOID;
+			else if(0 == strcmp(options_os_scan_timing, TIMING_SCAN_SNEAKY))
+				options_os_scan_flags |= OS_SCAN_OPTION_TIMIMG_SNEAKY;
+			else if(0 == strcmp(options_os_scan_timing, TIMING_SCAN_POLITE))
+				options_os_scan_flags |= OS_SCAN_OPTION_TIMIMG_POLITE;
+			else if(0 == strcmp(options_os_scan_timing, TIMING_SCAN_AGGRESSIVE))
+				options_os_scan_flags |= OS_SCAN_OPTION_TIMIMG_AGGRESSIVE;
+			else if(0 == strcmp(options_os_scan_timing, TIMING_SCAN_INSANE))
+				options_os_scan_flags |= OS_SCAN_OPTION_TIMIMG_INSANE;
+			else
+			{
+				printf(__FUNCTION__"(TIMING): you should not see this message!\n");
+				options_os_scan_flags |= OS_SCAN_OPTION_TIMIMG_NORMAL;
+			}
+			
 			break;
 		case OPTIONS_OS_SCAN_SCAN_SPECIFIED_PORTS:
 			if(options_os_scan_scan_specified_ports)
@@ -234,17 +259,38 @@ void apply_the_setting(int flags)
 			else
 				options_os_scan_flags &= ~OS_SCAN_OPTION_OSSCAN;
 			break;
+		case OPTIONS_OS_SCAN_RPC_SCAN:
+			if(options_os_scan_osscan)
+				options_os_scan_flags |= OS_SCAN_OPTION_RPC_SCAN;
+			else
+				options_os_scan_flags &= ~OS_SCAN_OPTION_RPC_SCAN;
+			break;
+		case OPTIONS_OS_SCAN_IDENTD_SCAN:
+			if(options_os_scan_osscan)
+				options_os_scan_flags |= OS_SCAN_OPTION_IDENTD_SCAN;
+			else
+				options_os_scan_flags &= ~OS_SCAN_OPTION_IDENTD_SCAN;
+			break;
 		default:
 			break;		
 	}
 }
 
+void apply_all_settings(void)
+{
+	int i;
+	
+	for(i = 0; i < OPTIONS_LAST_OPTION; i++)
+		apply_the_setting(i);
+}
 
 void apply_settings(GtkWidget *w, gpointer data)
 {
-	int retries, i = 0;
+	int retries;
 	char *temp;
 	
+	APPLY_CHECK_OPTION(os_scan_identd_scan, options_os_scan_identd_scan, OPTIONS_OS_SCAN_IDENTD_SCAN);
+	APPLY_CHECK_OPTION(os_scan_rpc_scan, options_os_scan_rpc_scan, OPTIONS_OS_SCAN_RPC_SCAN);
 	APPLY_CHECK_OPTION(os_scan_osscan, options_os_scan_osscan, OPTIONS_OS_SCAN_OSSCAN);
 	APPLY_CHECK_OPTION(os_scan_fastscan, options_os_scan_fastscan, OPTIONS_OS_SCAN_FASTSCAN);
 	APPLY_CHECK_OPTION(os_scan_dont_ping, options_os_scan_dont_ping, OPTIONS_OS_SCAN_DONT_PING);
@@ -268,12 +314,10 @@ void apply_settings(GtkWidget *w, gpointer data)
 		return;
 	}
 	options_discover_retries = retries;
-	apply_the_setting(OPTIONS_DISCOVER_RETRIES);
 	
 
 	temp = gtk_entry_get_text(GTK_ENTRY(gnome_entry_gtk_entry(GNOME_ENTRY(os_scan_ports))));
 	strncpy(options_os_scan_ports, temp, sizeof(options_os_scan_ports));
-	apply_the_setting(OPTIONS_OS_SCAN_PORTS);
 
 	temp = gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(os_scan_tcp_scan)->entry));
 	if(0 == strcmp(temp, TCP_SCAN_0))
@@ -288,13 +332,34 @@ void apply_settings(GtkWidget *w, gpointer data)
 		;
 	else
 	{
-		printf("you should not see this message!\n");
+		printf(__FUNCTION__"(): you should not see this message!\n");
 		temp = TCP_SCAN_DEFAULT;
 	}
 
-	printf("option %d '%s'\n", i, temp);	
 	strncpy(options_os_scan_tcp_scan, temp, sizeof(options_os_scan_tcp_scan));	
-	apply_the_setting(OPTIONS_OS_SCAN_TCP_SCAN);
+
+	temp = gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(os_scan_timing)->entry));
+	if(0 == strcmp(temp, TIMING_SCAN_NORMAL))
+		;
+	else if(0 == strcmp(temp, TIMING_SCAN_PARANOID))
+		;
+	else if(0 == strcmp(temp, TIMING_SCAN_SNEAKY))
+		;
+	else if(0 == strcmp(temp, TIMING_SCAN_POLITE))
+		;
+	else if(0 == strcmp(temp, TIMING_SCAN_AGGRESSIVE))
+		;
+	else if(0 == strcmp(temp, TIMING_SCAN_INSANE))
+		;
+	else
+	{
+		printf(__FUNCTION__"(): you should not see this message!\n");
+		temp = TIMING_SCAN_DEFAULT;
+	}
+
+	strncpy(options_os_scan_timing, temp, sizeof(options_os_scan_timing));	
+
+	apply_all_settings();
 }
 
 /*************************************************************************
@@ -360,8 +425,6 @@ service_list_t *services_make_dialog(service_list_t *service, char *name, int po
 	gtk_table_attach_defaults(GTK_TABLE(table), protocol_w, 1, 2, 2, 3);
 	gtk_table_attach_defaults(GTK_TABLE(table), string_w,   1, 2, 3, 4);
 
-	gtk_signal_connect(GTK_OBJECT(GTK_COMBO(string_w)->entry), "activate", click_ok_on_gnome_dialog, dialog);
-
 	gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(dialog)->vbox), table, FALSE, FALSE, 5);
 
 	label = gtk_label_new(FLAG_CHARACTER_DIRECTIONS);
@@ -385,6 +448,12 @@ service_list_t *services_make_dialog(service_list_t *service, char *name, int po
 	gtk_entry_set_text(GTK_ENTRY(gnome_entry_gtk_entry(GNOME_ENTRY(string_w))), str );
 	gtk_combo_set_value_in_list(GTK_COMBO(protocol_w), (protocol==6 ?0:1), FALSE);
 
+	gtk_widget_grab_focus(GTK_COMBO(name_w)->entry);
+	gtk_signal_connect(GTK_OBJECT(GTK_COMBO(name_w)->entry),  "activate", GTK_SIGNAL_FUNC(gtk_widget_focus_me), GTK_COMBO(port_w)->entry);
+	gtk_signal_connect(GTK_OBJECT(GTK_COMBO(port_w)->entry),  "activate", GTK_SIGNAL_FUNC(gtk_widget_focus_me), GTK_COMBO(protocol_w)->entry);
+	gtk_signal_connect(GTK_OBJECT(GTK_COMBO(protocol_w)->entry),  "activate", GTK_SIGNAL_FUNC(gtk_widget_focus_me), GTK_COMBO(string_w)->entry);
+	gtk_signal_connect(GTK_OBJECT(GTK_COMBO(string_w)->entry), "activate", click_ok_on_gnome_dialog, dialog);
+
 	gtk_widget_show( name_w);
 	gtk_widget_show( port_w);
 	gtk_widget_show( protocol_w);
@@ -553,6 +622,10 @@ service_list_t *probe_ports_make_dialog(service_list_t *service, char *name, int
 	gtk_table_attach_defaults(GTK_TABLE(table), protocol_w, 1, 2, 2, 3);
 	gtk_table_attach_defaults(GTK_TABLE(table), string_w,   1, 2, 3, 4);
 
+	gtk_widget_grab_focus(GTK_COMBO(name_w)->entry);
+	gtk_signal_connect(GTK_OBJECT(GTK_COMBO(name_w)->entry),  "activate", GTK_SIGNAL_FUNC(gtk_widget_focus_me), GTK_COMBO(port_w)->entry);
+	gtk_signal_connect(GTK_OBJECT(GTK_COMBO(port_w)->entry),  "activate", GTK_SIGNAL_FUNC(gtk_widget_focus_me), GTK_COMBO(protocol_w)->entry);
+	gtk_signal_connect(GTK_OBJECT(GTK_COMBO(protocol_w)->entry),  "activate", GTK_SIGNAL_FUNC(gtk_widget_focus_me), GTK_COMBO(string_w)->entry);
 	gtk_signal_connect(GTK_OBJECT(GTK_COMBO(string_w)->entry), "activate", click_ok_on_gnome_dialog, dialog);
 
 	gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(dialog)->vbox), table, FALSE, FALSE, 5);
@@ -980,7 +1053,7 @@ void do_settings(GtkWidget *widget, gpointer data)
 	gtk_notebook_set_tab_pos(GTK_NOTEBOOK(notebook), GTK_POS_TOP);
 	gtk_notebook_set_show_border(GTK_NOTEBOOK(notebook), FALSE);
 	gtk_table_attach_defaults(GTK_TABLE(table),notebook,0,2,0,7);
-	gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(dialog)->vbox), table, FALSE, FALSE, 5);
+	gtk_box_pack_start(GTK_BOX(GNOME_DIALOG(dialog)->vbox), table, FALSE, FALSE, 0);
 	
 /*
  * Agent Settings Tab
@@ -1020,8 +1093,8 @@ void do_settings(GtkWidget *widget, gpointer data)
 	frame = gtk_frame_new("Discover Settings");
 	vbox = gtk_vbox_new(FALSE, 5);
 	gtk_container_add(GTK_CONTAINER(frame), vbox);
-	gtk_box_pack_start(GTK_BOX(vbox), table3, FALSE, FALSE, 5);
-	gtk_box_pack_start(GTK_BOX(table2), frame, FALSE, FALSE, 5);
+	gtk_box_pack_start(GTK_BOX(vbox), table3, FALSE, FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(table2), frame, FALSE, FALSE, 0);
 	gtk_widget_show(frame);
 	gtk_widget_show(vbox);
 
@@ -1034,6 +1107,12 @@ void do_settings(GtkWidget *widget, gpointer data)
 	os_scan_udp_scan = gtk_check_button_new_with_label("Enable UDP port scanning");
 	SET_CHECK_OPTION(os_scan_udp_scan, options_os_scan_udp_scan);
 
+	os_scan_rpc_scan = gtk_check_button_new_with_label("Enable RPC scanning");
+	SET_CHECK_OPTION(os_scan_rpc_scan, options_os_scan_rpc_scan);
+
+	os_scan_identd_scan = gtk_check_button_new_with_label("Enable identd scanning");
+	SET_CHECK_OPTION(os_scan_identd_scan, options_os_scan_identd_scan);
+
 	os_scan_fastscan = gtk_check_button_new_with_label("Enable Fastscan");
 	SET_CHECK_OPTION(os_scan_fastscan, options_os_scan_fastscan);
 	gtk_signal_connect(GTK_OBJECT(os_scan_fastscan), "toggled", (GtkSignalFunc)fastscan_toggle, NULL);
@@ -1044,14 +1123,14 @@ void do_settings(GtkWidget *widget, gpointer data)
 	frame = gtk_frame_new("Nmap settings");
 	vbox = gtk_vbox_new(FALSE, 5);
 	gtk_container_add(GTK_CONTAINER(frame), vbox);
-	gtk_box_pack_start(GTK_BOX(vbox), os_scan_osscan, FALSE, FALSE, 5);
-	gtk_box_pack_start(GTK_BOX(vbox), os_scan_udp_scan, FALSE, FALSE, 5);
+	gtk_box_pack_start(GTK_BOX(vbox), os_scan_osscan, FALSE, FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(vbox), os_scan_udp_scan, FALSE, FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(vbox), os_scan_rpc_scan, FALSE, FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(vbox), os_scan_identd_scan, FALSE, FALSE, 0);
 
 
 	label = gtk_label_new("Select TCP Scan Type:");
 	os_scan_tcp_scan = gtk_combo_new();
-	
-	sprintf(char_ptr, "%d", i);
 	comboList = NULL;
 	comboList = g_list_append(comboList, TCP_SCAN_0);
 	comboList = g_list_append(comboList, TCP_SCAN_1);
@@ -1067,11 +1146,31 @@ void do_settings(GtkWidget *widget, gpointer data)
 	gtk_table_attach_defaults(GTK_TABLE(table3),os_scan_tcp_scan,1,2,0,1);
 	gtk_widget_show(table3);
 	gtk_widget_show(label);
-	gtk_box_pack_start(GTK_BOX(vbox), table3, FALSE, FALSE, 5);
+	gtk_box_pack_start(GTK_BOX(vbox), table3, FALSE, FALSE, 0);
 
-	gtk_box_pack_start(GTK_BOX(vbox), os_scan_dont_ping, FALSE, FALSE, 5);
-	gtk_box_pack_start(GTK_BOX(vbox), os_scan_fastscan, FALSE, FALSE, 5);
-	gtk_box_pack_start(GTK_BOX(vbox), os_scan_scan_specified_ports, FALSE, FALSE, 5);
+	label = gtk_label_new("Select Scan Timing:");
+	os_scan_timing = gtk_combo_new();
+	comboList = NULL;
+	comboList = g_list_append(comboList, TIMING_SCAN_NORMAL);
+	comboList = g_list_append(comboList, TIMING_SCAN_PARANOID);
+	comboList = g_list_append(comboList, TIMING_SCAN_SNEAKY);
+	comboList = g_list_append(comboList, TIMING_SCAN_POLITE);
+	comboList = g_list_append(comboList, TIMING_SCAN_AGGRESSIVE);
+	comboList = g_list_append(comboList, TIMING_SCAN_INSANE);
+	gtk_combo_set_popdown_strings(GTK_COMBO(os_scan_timing),comboList);
+	gtk_combo_set_use_arrows_always(GTK_COMBO(os_scan_timing),1);
+	gtk_entry_set_text(GTK_ENTRY(GTK_COMBO(os_scan_timing)->entry), options_os_scan_timing);
+	gtk_entry_set_editable(GTK_ENTRY(GTK_COMBO(os_scan_timing)->entry), FALSE);
+	table3 = gtk_table_new(1,2,TRUE);
+	gtk_table_attach_defaults(GTK_TABLE(table3),label,0,1,0,1);
+	gtk_table_attach_defaults(GTK_TABLE(table3),os_scan_timing,1,2,0,1);
+	gtk_widget_show(table3);
+	gtk_widget_show(label);
+	gtk_box_pack_start(GTK_BOX(vbox), table3, FALSE, FALSE, 0);
+
+	gtk_box_pack_start(GTK_BOX(vbox), os_scan_dont_ping, FALSE, FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(vbox), os_scan_fastscan, FALSE, FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(vbox), os_scan_scan_specified_ports, FALSE, FALSE, 0);
 
 	os_scan_ports = gnome_entry_new("os_scan_ports");
 
@@ -1083,14 +1182,17 @@ void do_settings(GtkWidget *widget, gpointer data)
 	gtk_table_attach_defaults(GTK_TABLE(table3),os_scan_ports,1,2,0,1);
 	gtk_widget_show(table3);
 	gtk_widget_show(os_scan_ports_label);
-	gtk_box_pack_start(GTK_BOX(vbox), table3, FALSE, FALSE, 5);
+	gtk_box_pack_start(GTK_BOX(vbox), table3, FALSE, FALSE, 0);
 
-	gtk_box_pack_start(GTK_BOX(table2), frame, FALSE, FALSE, 5);
+	gtk_box_pack_start(GTK_BOX(table2), frame, FALSE, FALSE, 0);
 	gtk_widget_show(frame);
 	gtk_widget_show(vbox);
 	gtk_widget_show(os_scan_osscan);
 	gtk_widget_show(os_scan_udp_scan);
+	gtk_widget_show(os_scan_rpc_scan);
+	gtk_widget_show(os_scan_identd_scan);
 	gtk_widget_show(os_scan_tcp_scan);
+	gtk_widget_show(os_scan_timing);
 	gtk_widget_show(os_scan_scan_specified_ports);
 	gtk_widget_show(os_scan_ports);
 	gtk_widget_show(os_scan_ports_label);
@@ -1140,24 +1242,24 @@ void do_settings(GtkWidget *widget, gpointer data)
 	frame = gtk_frame_new("Tool Options");
 	vbox = gtk_vbox_new(FALSE, 5);
 	gtk_container_add(GTK_CONTAINER(frame), vbox);
-	gtk_box_pack_start(GTK_BOX(vbox), operating_system_check, FALSE, FALSE, 5);
-	gtk_box_pack_start(GTK_BOX(vbox), reverse_dns, FALSE, FALSE, 5);
-	gtk_box_pack_start(GTK_BOX(vbox), map, FALSE, FALSE, 5);
-	gtk_box_pack_start(GTK_BOX(vbox), probe_ports, FALSE, FALSE, 5);
-	gtk_box_pack_start(GTK_BOX(vbox), rescan_at_startup, FALSE, FALSE, 5);
-	gtk_box_pack_start(GTK_BOX(table2), frame, FALSE, FALSE, 5);
+	gtk_box_pack_start(GTK_BOX(vbox), operating_system_check, FALSE, FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(vbox), reverse_dns, FALSE, FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(vbox), map, FALSE, FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(vbox), probe_ports, FALSE, FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(vbox), rescan_at_startup, FALSE, FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(table2), frame, FALSE, FALSE, 0);
 	gtk_widget_show(frame);
 	gtk_widget_show(vbox);
 		
 	frame = gtk_frame_new("Look and Feel");
 	vbox = gtk_vbox_new(FALSE, 5);
 	gtk_container_add(GTK_CONTAINER(frame), vbox);
-	gtk_box_pack_start(GTK_BOX(vbox), confirm_delete, FALSE, FALSE, 5);
-	gtk_box_pack_start(GTK_BOX(vbox), use_ip_for_label, FALSE, FALSE, 5);
-	gtk_box_pack_start(GTK_BOX(vbox), save_changes_on_exit, FALSE, FALSE, 5);
-	gtk_box_pack_start(GTK_BOX(vbox), move_stuff_live, FALSE, FALSE, 5);
-	gtk_box_pack_start(GTK_BOX(vbox), use_ip_for_merged_ports, FALSE, FALSE, 5);
-	gtk_box_pack_start(GTK_BOX(table2), frame, FALSE, FALSE, 5);
+	gtk_box_pack_start(GTK_BOX(vbox), confirm_delete, FALSE, FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(vbox), use_ip_for_label, FALSE, FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(vbox), save_changes_on_exit, FALSE, FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(vbox), move_stuff_live, FALSE, FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(vbox), use_ip_for_merged_ports, FALSE, FALSE, 0);
+	gtk_box_pack_start(GTK_BOX(table2), frame, FALSE, FALSE, 0);
 	gtk_widget_show(frame);
 	gtk_widget_show(vbox);
 
@@ -1178,7 +1280,7 @@ void do_settings(GtkWidget *widget, gpointer data)
  * Services Settings Tab
  */
 	label = gtk_label_new("Services");
-	table2 = gtk_table_new(6, 3, TRUE);
+	table2 = gtk_table_new(9, 3, TRUE);
 
 	gtk_notebook_append_page(GTK_NOTEBOOK(notebook),table2,label);
 	
@@ -1198,7 +1300,7 @@ void do_settings(GtkWidget *widget, gpointer data)
 
 	gtk_container_add(GTK_CONTAINER(scrolled_window), services_clist);
 
-	gtk_table_attach_defaults(GTK_TABLE(table2), scrolled_window, 0, 3, 0, 5);
+	gtk_table_attach_defaults(GTK_TABLE(table2), scrolled_window, 0, 3, 0, 8);
 	
 	{
 		service_list_t *p;
@@ -1241,9 +1343,9 @@ void do_settings(GtkWidget *widget, gpointer data)
 	gtk_signal_connect(GTK_OBJECT(services_delete), "clicked", GTK_SIGNAL_FUNC(services_do_delete), NULL);
 	 
 
-	gtk_table_attach(GTK_TABLE(table2), button,          0, 1, 5, 6, GTK_FILL, GTK_FILL, 5, 5);
-	gtk_table_attach(GTK_TABLE(table2), services_edit,   1, 2, 5, 6, GTK_FILL, GTK_FILL, 5, 5);
-	gtk_table_attach(GTK_TABLE(table2), services_delete, 2, 3, 5, 6, GTK_FILL, GTK_FILL, 5, 5);
+	gtk_table_attach(GTK_TABLE(table2), button,          0, 1, 8, 9, GTK_FILL, GTK_FILL, 5, 5);
+	gtk_table_attach(GTK_TABLE(table2), services_edit,   1, 2, 8, 9, GTK_FILL, GTK_FILL, 5, 5);
+	gtk_table_attach(GTK_TABLE(table2), services_delete, 2, 3, 8, 9, GTK_FILL, GTK_FILL, 5, 5);
 	
 	gtk_widget_show(services_clist);
 	gtk_widget_show(button);
@@ -1305,7 +1407,7 @@ void do_settings(GtkWidget *widget, gpointer data)
  * OS Pixmaps Tab
  */
 	label = gtk_label_new("OS Pixmaps");
-	table2 = gtk_table_new(6, 3, TRUE);
+	table2 = gtk_table_new(9, 3, TRUE);
 
 	gtk_notebook_append_page(GTK_NOTEBOOK(notebook),table2,label);
 	
@@ -1324,7 +1426,7 @@ void do_settings(GtkWidget *widget, gpointer data)
 
 	gtk_container_add(GTK_CONTAINER(scrolled_window), os_pixmaps_clist);
 
-	gtk_table_attach_defaults(GTK_TABLE(table2), scrolled_window, 0, 3, 0, 5);
+	gtk_table_attach_defaults(GTK_TABLE(table2), scrolled_window, 0, 3, 0, 8);
 	
 	{
 		os_pixmap_list_t *p;
@@ -1379,9 +1481,9 @@ void do_settings(GtkWidget *widget, gpointer data)
 	gtk_signal_connect(GTK_OBJECT(os_pixmaps_delete), "clicked", GTK_SIGNAL_FUNC(os_pixmaps_do_delete), NULL);
 	 
 
-	gtk_table_attach(GTK_TABLE(table2), button,            0, 1, 5, 6, GTK_FILL, GTK_FILL, 5, 5);
-	gtk_table_attach(GTK_TABLE(table2), os_pixmaps_edit,   1, 2, 5, 6, GTK_FILL, GTK_FILL, 5, 5);
-	gtk_table_attach(GTK_TABLE(table2), os_pixmaps_delete, 2, 3, 5, 6, GTK_FILL, GTK_FILL, 5, 5);
+	gtk_table_attach(GTK_TABLE(table2), button,            0, 1, 8, 9, GTK_FILL, GTK_FILL, 5, 5);
+	gtk_table_attach(GTK_TABLE(table2), os_pixmaps_edit,   1, 2, 8, 9, GTK_FILL, GTK_FILL, 5, 5);
+	gtk_table_attach(GTK_TABLE(table2), os_pixmaps_delete, 2, 3, 8, 9, GTK_FILL, GTK_FILL, 5, 5);
 	
 	gtk_widget_show(os_pixmaps_clist);
 	gtk_widget_show(button);
